[![Build Status](https://api.travis-ci.org/kolypto/py-mailem.png?branch=master)](https://travis-ci.org/kolypto/py-mailem)

{% macro argspec(args) -%}
{% for a in args %}{% if a.doc %}* `{{ a.name }}`: {{ a.doc }}{% endif %}
{% endfor %}
{%- endmacro %}

{% macro clsdoc(cls) -%}
```python
{{ cls.signature|wordwrap(40)|indent(1+cls.name|length) }}
```

{{ cls.doc }}

{{ argspec(cls.args) }}
{%- endmacro %}

{% macro fdoc(func) -%}
{{ func.doc }}

{{ argspec(func.args) }}
{%- endmacro %}

Mail'Em
=======

{{ mailem.doc }}



Table of Contents
=================



Message
=======
{{ clsdoc(Message) }}

Attachment
----------
{{ clsdoc(Attachment) }}

ImageAttachment
---------------
{{ clsdoc(ImageAttachment) }}

Postman
=======
{{ clsdoc(Postman.cls) }}

### `Postman.{{ Postman.attrs.connect.signature }}`
{{ Postman.attrs.connect.doc }}

### `Postman.{{ Postman.attrs.loopback.signature }}`
{{ Postman.attrs.loopback.doc }}

Connection
==========

{{ connection.doc }}

SMTPConnection
--------------
{{ clsdoc(SMTPConnection) }}

LoopbackConnection
------------------
{{ clsdoc(LoopbackConnection) }}

Template
========
{{ clsdoc(Template.cls) }}

### `Template.{{ Template.attrs.set_renderer.signature }}`
{{ fdoc(Template.attrs.set_renderer) }}

### `Template.{{ Template.attrs.__call__.signature }}`
{{ fdoc(Template.attrs.__call__) }}

### `Template.{{ Template.attrs.from_directory.signature }}`
{{ fdoc(Template.attrs.from_directory) }}
